@@ -1,7 +1,5 @@
-// Copyright (C) 2009 NICTA
-// 
-// Authors:
-// - Conrad Sanderson (conradsand at ieee dot org)
+// Copyright (C) 2008-2011 NICTA (www.nicta.com.au)
+// Copyright (C) 2008-2011 Conrad Sanderson
 // 
 // This file is part of the Armadillo C++ library.
 // It is provided without any warranty of fitness
@@ -19,15 +17,14 @@
 //! \brief
 //! Delayed 'minimum values' operation.
 //! The dimension, along which the minima are found, is set via 'dim'.
-//! For dim = 0, the maximum value of each column is found (i.e. searches by traversing across rows).
-//! For dim = 1, the maximum value of each row is found (i.e. searches by traversing across columns).
+//! For dim = 0, the minimum value of each column is found (i.e. searches by traversing across rows).
+//! For dim = 1, the minimum value of each row is found (i.e. searches by traversing across columns).
 //! The default is dim = 0.
-//! NOTE: This function works differently than in Matlab/Octave.
 
 template<typename T1>
-inline
+arma_inline
 const Op<T1, op_min>
-min(const Base<typename T1::elem_type,T1>& X, const u32 dim = 0)
+min(const Base<typename T1::elem_type,T1>& X, const uword dim = 0)
   {
   arma_extra_debug_sigprint();
   
@@ -38,14 +35,17 @@ min(const Base<typename T1::elem_type,T1>& X, const u32 dim = 0)
 //! Immediate 'find the minimum value in a row vector' operation
 template<typename eT>
 inline
+arma_warn_unused
 eT
 min(const Row<eT>& A)
   {
   arma_extra_debug_sigprint();
   
-  arma_debug_check( (A.n_elem == 0), "min(): given vector has no elements" );
+  const uword A_n_elem = A.n_elem;
   
-  return op_min::direct_min(A.mem, A.n_elem);
+  arma_debug_check( (A_n_elem == 0), "min(): given object has no elements" );
+  
+  return op_min::direct_min(A.mem, A_n_elem);
   }
 
 
@@ -53,14 +53,17 @@ min(const Row<eT>& A)
 //! Immediate 'find the minimum value in a column vector'
 template<typename eT>
 inline
+arma_warn_unused
 eT
 min(const Col<eT>& A)
   {
   arma_extra_debug_sigprint();
   
-  arma_debug_check( (A.n_elem == 0), "min(): given vector has no elements" );
+  const uword A_n_elem = A.n_elem;
   
-  return op_min::direct_min(A.mem, A.n_elem);
+  arma_debug_check( (A_n_elem == 0), "min(): given object has no elements" );
+  
+  return op_min::direct_min(A.mem, A_n_elem);
   }
 
 
@@ -70,6 +73,7 @@ min(const Col<eT>& A)
 //! invoked, for example, by: min(min(A))
 template<typename T1>
 inline
+arma_warn_unused
 typename T1::elem_type
 min(const Op<T1, op_min>& in)
   {
@@ -81,9 +85,11 @@ min(const Op<T1, op_min>& in)
   const unwrap<T1> tmp1(in.m);
   const Mat<eT>& X = tmp1.M;
   
-  arma_debug_check( (X.n_elem == 0), "min(): given matrix has no elements" );
+  const uword X_n_elem = X.n_elem;
   
-  return op_min::direct_min(X.mem, X.n_elem);
+  arma_debug_check( (X_n_elem == 0), "min(): given object has no elements" );
+  
+  return op_min::direct_min(X.mem, X_n_elem);
   }
 
 
@@ -91,7 +97,7 @@ min(const Op<T1, op_min>& in)
 template<typename T1>
 inline
 const Op< Op<T1, op_min>, op_min>
-min(const Op<T1, op_min>& in, const u32 dim)
+min(const Op<T1, op_min>& in, const uword dim)
   {
   arma_extra_debug_sigprint();
   
@@ -102,12 +108,13 @@ min(const Op<T1, op_min>& in, const u32 dim)
 
 template<typename eT>
 inline
+arma_warn_unused
 eT
 min(const subview_row<eT>& A)
   {
   arma_extra_debug_sigprint();
   
-  arma_debug_check( (A.n_elem == 0), "min(): given vector has no elements" );
+  arma_debug_check( (A.n_elem == 0), "min(): given object has no elements" );
   
   return op_min::direct_min(A);
   }
@@ -116,26 +123,28 @@ min(const subview_row<eT>& A)
 
 template<typename eT>
 inline
+arma_warn_unused
 eT
 min(const subview_col<eT>& A)
   {
   arma_extra_debug_sigprint();
   
-  arma_debug_check( (A.n_elem == 0), "min(): given vector has no elements" );
+  arma_debug_check( (A.n_elem == 0), "min(): given object has no elements" );
   
-  return op_min::direct_min(A);
+  return op_min::direct_min(A.colptr(0), A.n_rows);
   }
 
 
 
 template<typename eT>
 inline
+arma_warn_unused
 eT
 min(const diagview<eT>& A)
   {
   arma_extra_debug_sigprint();
   
-  arma_debug_check( (A.n_elem == 0), "min(): given vector has no elements" );
+  arma_debug_check( (A.n_elem == 0), "min(): given object has no elements" );
   
   return op_min::direct_min(A);
   }
@@ -144,19 +153,39 @@ min(const diagview<eT>& A)
 
 template<typename eT>
 inline
+arma_warn_unused
 eT
 min(const Op<subview<eT>, op_min>& in)
   {
   arma_extra_debug_sigprint();
-  arma_extra_debug_print("max(): two consecutive max() calls detected");
+  arma_extra_debug_print("min(): two consecutive min() calls detected");
   
   const subview<eT>& X = in.m;
   
-  arma_debug_check( (X.n_elem == 0), "max(): given matrix has no elements" );
+  arma_debug_check( (X.n_elem == 0), "min(): given object has no elements" );
   
   return op_min::direct_min(X);
   }
 
 
 
+template<typename eT, typename T1>
+inline
+arma_warn_unused
+eT
+min(const subview_elem1<eT,T1>& A)
+  {
+  arma_extra_debug_sigprint();
+  
+  const Mat<eT> X(A);
+  
+  const uword X_n_elem = X.n_elem;
+  
+  arma_debug_check( (X_n_elem == 0), "min(): given object has no elements" );
+  
+  return op_min::direct_min(X.mem, X_n_elem);
+  }
+
+
+
 //! @}
